# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Mozilla Corporation Code.
#
# The Initial Developer of the Original Code is
# Clint Talbert.
# Portions created by the Initial Developer are Copyright (C) 2007
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#  Clint Talbert <ctalbert@mozilla.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

from optparse import OptionParser
import platform
import subprocess
import re
import time
import string
import os
import shutil
import zipfile

isDMG = re.compile(".*\.dmg")
isTARBZ = re.compile(".*\.tar\.bz")
isTARGZ = re.compile(".*\.tar\.gz")
isZIP = re.compile(".*\.zip")
isEXE = re.compile(".*\.exe")
_mozInstall_debug = False

def debug(s):
  if _mozInstall_debug:
    print "DEBUG: " + s

cygwinmatch = re.compile(".*cygwin.*", re.I)

# Copied from mozilla/testing/release/minotaur/getOSInfo.py - want this class
# to be self-sufficient without requiring another "non-standard" python lib,
# so duplicated this function from there.
def getPlatform():
  # On Vista, python reports "Microsoft" and on cygwin shells it can report
  # several different strings that contain the word "cygwin"
  if platform.system() == "Microsoft" or cygwinmatch.search(platform.system()):
    return "Windows"
  else:
    return platform.system()

# This code is lifted directly from the buildbot code. That we use here. I
# can't call it in buildbot without importing a trillion other things.
# Original Source: http://mxr.mozilla.org/mozilla/source/tools/buildbot/buildbot/slave/commands.py#59
def rmdirRecursive(dir):
  """This is a replacement for shutil.rmtree that works better under
  windows. Thanks to Bear at the OSAF for the code."""
  if not os.path.exists(dir):
      return

  if os.path.islink(dir):
      os.remove(dir)
      return

  # Verify the directory is read/write/execute for the current user
  os.chmod(dir, 0700)

  for name in os.listdir(dir):
      full_name = os.path.join(dir, name)
      # on Windows, if we don't have write permission we can't remove
      # the file/directory either, so turn that on
      if os.name == 'nt':
          if not os.access(full_name, os.W_OK):
              # I think this is now redundant, but I don't have an NT
              # machine to test on, so I'm going to leave it in place
              # -warner
              os.chmod(full_name, 0600)

      if os.path.isdir(full_name):
          rmdirRecursive(full_name)
      else:
          os.chmod(full_name, 0700)
          os.remove(full_name)
  os.rmdir(dir)

class MozUninstaller:
  def __init__(self, **kwargs):
    debug("uninstall constructor")
    assert (kwargs['dest'] != "" and kwargs['dest'] != None)
    assert (kwargs['productName'] != "" and kwargs['productName'] != None)
    assert (kwargs['branch'] != "" and kwargs['dest'] != None)
    self.dest = kwargs['dest']
    self.productName = kwargs['productName']
    self.branch = kwargs['branch']

    # Handle the case where we haven't installed yet
    if not os.path.exists(self.dest):
      return

    if getPlatform() == "Windows":
      try:
        self.doWindowsUninstall()
      except:
        debug("Windows Uninstall threw - not overly urgent or worrisome")
    if os.path.exists(self.dest):
      try:
        os.rmdir(self.dest)
      except OSError:
        # Directories are still there - kill them all!
        rmdirRecursive(self.dest)


  def doWindowsUninstall(self):
    debug("do windowsUninstall")
    if self.branch == "1.8.0":
      uninstexe = self.dest + "/uninstall/uninstall.exe"
      uninstini = self.dest + "/uninstall/uninstall.ini"
      debug("uninstexe: " + uninstexe)
      debug("uninstini: " + uninstini)
      if os.path.exists(uninstexe):
        # modify uninstall.ini to run silently
        debug("modifying uninstall.ini")
        args = "sed -i.bak 's/Run Mode=Normal/Run Mode=Silent/' " + uninstini
        proc = subprocess.Popen(args, shell=True)
        # Todo handle error
        proc.wait()
        proc = subprocess.Popen(uninstexe, shell=True)
        proc.wait()
    elif self.branch == "1.8.1" or self.branch == "1.8" or self.branch == "1.9":
      debug("we are in 1.8 uninstall land")
      uninst = self.dest + "/uninstall/uninst.exe"
      helper = self.dest + "/uninstall/helper.exe"
      debug("uninst: " + uninst)
      debug("helper: " + helper)

      if os.path.exists(helper):
        debug("helper exists")
        args = helper + " /S /D=" + os.path.normpath(self.dest)
        debug("running helper with args: " + args)
        proc = subprocess.Popen(args, shell=True)
        proc.wait()
      elif os.path.exists(uninst):
        args = uninst + " /S /D=" + os.path.normpath(self.dest)
        debug("running uninst with args: " + args)
        proc = subprocess.Popen(args, shell=True)
        proc.wait()
      else:
        uninst = self.dest + "/" + self.product + "/uninstall/uninstaller.exe"
        args = uninst + " /S /D=" + os.path.normpath(self.dest)
        debug("running uninstaller with args: " + args)
        proc = subprocess.Popen(args, shell=True)
        proc.wait()
    time.sleep(10)

class MozInstaller:
  def __init__(self, **kwargs):
    debug("install constructor!")
    assert (kwargs['dest'] != "" and kwargs['dest'] != None)
    assert (kwargs['src'] != "" and kwargs['src'] != None)
    #assert (kwargs['productName'] != "" and kwargs['productName'] != None)
    #assert (kwargs['branch'] != "" and kwargs['branch'] != None)
    self.src = kwargs['src']
    self.dest = kwargs['dest']
    #self.productName = kwargs['productName']
    #self.branch = kwargs['branch']
    #debug("running uninstall")
    #uninstaller = MozUninstaller(dest = self.dest, productName = self.productName,
     #                            branch = self.branch)

    if isDMG.match(self.src):
      self.installDmg()
    elif isTARBZ.match(self.src):
      self.installTarBz()
    elif isTARGZ.match(self.src):
      self.installTarGz()
    elif isZIP.match(self.src):
      self.installZip()
    elif isEXE.match(self.src):
      self.installExe()

  # Simple utility function to get around python's path module's inability
  # to understand ~/... style paths
  def normalizePath(self, path):
    if path[0] == "~":
      path = os.path.expanduser(path)

    debug("NORMALIZE: path: " + path)

    try:
      if not os.path.exists(path):
        os.makedirs(path)
    except:
      # TODO: Better catch and error message
      print "Error creating destination directory " + path
    return path

  def installDmg(self):
    # Ensure our destination directory exists
    self.dest = self.normalizePath(self.dest)

    # undmg code from Jesse
    mountpoint = os.path.join(self.dest, "MOUNTEDDMG")
    devnull = open(os.devnull, "w")
    subprocess.check_call(["hdiutil", "attach", "-quiet", "-mountpoint", mountpoint, self.src], stdout=devnull)
    try:
      #while not os.path.exists(mountpoint + "/MinefieldDebug.app"):
      #  print "waiting for disk image"
      #  time.sleep(1)
      shutil.copytree(os.path.join(mountpoint, "Minefield.app"), os.path.join(self.dest, "Minefield.app"))
    finally:
      subprocess.check_call(["hdiutil", "detach", mountpoint], stdout=devnull)
      #shutil.rmtree(mountpoint)

  def installTarBz(self):
    # Ensure our destination directory exists
    self.dest = self.normalizePath(self.dest)
    self.unTar("-jxf")

  def installTarGz(self):
    # Ensure our destination directory exists
    self.dest = self.normalizePath(self.dest)
    self.unTar("-zxf")

  def unTar(self, tarArgs):
    args = "tar " + tarArgs + " " + self.src + " -C " + self.dest
    proc = subprocess.Popen(args, shell=True)
    proc.wait()
    #TODO: throw stderr

  def installZip(self):
    self.dest = self.normalizePath(self.dest)
    #TODO: throw stderr
    zipped = zipfile.ZipFile(self.src)
    try:
        zipped.extractall(self.dest)
    except:
        args = ["unzip", "-o", "-q", "-d", self.dest, self.src]
        proc = subprocess.Popen(args)
        proc.wait()
        
  def installExe(self):
    debug("running installEXE")
    args = self.src + " "
    if self.branch == "1.8.0":
      args += "-ms -hideBanner -dd " + self.dest
    else:
      debug("running install exe for 1.8.1")
      args += "/S /D=" + os.path.normpath(self.dest)
    # Do we need a shell=True here?
    proc = subprocess.Popen(args)
    proc.wait()
    # TODO: throw stderr

# Enable it to be called from the command line with the options
if __name__ == "__main__":
  parser = OptionParser()
  parser.add_option("-s", "--Source", dest="src",
                   help="Installation Source File (whatever was downloaded) -\
                         accepts Zip, Exe, Tar.Bz, Tar.Gz, and DMG",
                   metavar="SRC_FILE")
  parser.add_option("-d", "--Destination", dest="dest",
                    help="Directory to install the build into", metavar="DEST")
  parser.add_option("-b", "--Branch", dest="branch",
                    help="Branch the build is from must be one of: 1.8.0|1.8|\
                          1.9", metavar="BRANCH")
  parser.add_option("-p", "--Product", dest="product",
                    help="Product name - optional should be all lowercase if\
                         specified: firefox, thunderbird, etc",
                    metavar="PRODUCT")
  parser.add_option("-o", "--Operation", dest="op",
                    help="The operation you would like the script to perform.\
                         Should be either install (i) or uninstall (u) or delete\
                          (d) to recursively delete the directory specified in dest",
                    metavar="OP")

  (options, args) = parser.parse_args()

  # Run it
  if string.upper(options.op) == "INSTALL" or string.upper(options.op) == "I":
    installer = MozInstaller(src = options.src, dest = options.dest,
                             branch = options.branch, productName = options.product)
  elif string.upper(options.op) == "UNINSTALL" or string.upper(options.op) == "U":
    uninstaller = MozUninstaller(dest = options.dest, branch = options.branch,
                                 productName = options.product)
  elif string.upper(options.op) == "DELETE" or string.upper(options.op) == "D":
    rmdirRecursive(options.dest)



#!/usr/bin/env python
import datetime
import sys
from optparse import OptionParser
from runnightly import NightlyRunner
from utils import strsplit, get_date

class Bisector():
    def __init__(self, runner): 
        self.runner = runner
        self.currAppInfo = ''
        self.currDate = ''

    def bisect(self, goodDate, badDate):
        midDate = goodDate + (badDate - goodDate) / 2
        if midDate == badDate or midDate == goodDate:
            print "\n\nLast good nightly: " + str(goodDate) + " First bad nightly: " + str(badDate) + "\n"
            print "Pushlog: " + self.getPushlogUrl(goodDate, badDate) + "\n"
            sys.exit()
    
        # run the nightly from that date
        dest = self.runner.start(midDate)
        while not dest:
            midDate += datetime.timedelta(days=1)
            if midDate == badDate:
                print "\n\nLast good nightly: " + str(goodDate) + " First bad nightly: " + str(badDate) + "\n"
                print "Pushlog: " + self.getPushlogUrl(goodDate, badDate) + "\n"
                sys.exit()
            dest = self.runner.start(midDate)
        
        self.prevAppInfo = self.currAppInfo
        self.prevDate = self.currDate
        self.currAppInfo = self.runner.getAppInfo()
        self.currDate = midDate

        # wait for them to call it 'good' or 'bad'
        verdict = ""
        while verdict != 'good' and verdict != 'bad' and verdict != 'b' and verdict != 'g':
            verdict = raw_input("Was this nightly good or bad? (type 'good' or 'bad' and press Enter): ")

        self.runner.stop()
        if verdict == 'good' or verdict == 'g':
            self.bisect(midDate, badDate)
        else:
            self.bisect(goodDate, midDate)

    def getPushlogUrl(self, goodDate, badDate):
        if not self.currAppInfo or not self.prevAppInfo:
            if self.currAppInfo:
                (repo, chset) = self.currAppInfo
            elif self.prevAppInfo:
                (repo, chset) = self.prevAppInfo
            else:
                repo = ''

            return repo + "/pushloghtml?startdate=" + str(goodDate) + "&enddate=" + str(badDate)

        (repo, chset1) = self.currAppInfo
        (repo, chset2) = self.prevAppInfo
        if self.currDate > self.prevDate:
            return repo + "/pushloghtml?fromchange=" + chset2 + "&tochange=" + chset1
        return repo + "/pushloghtml?fromchange=" + chset1 + "&tochange=" + chset2
        
def cli():
    parser = OptionParser()
    parser.add_option("-b", "--bad", dest="bad_date",help="first known bad nightly build, default is today",
                      metavar="YYYY-MM-DD", default=str(datetime.date.today()))
    parser.add_option("-g", "--good", dest="good_date",help="last known good nightly build",
                      metavar="YYYY-MM-DD", default=None)
    parser.add_option("-e", "--addons", dest="addons",help="list of addons to install", metavar="PATH1,PATH2", default="")
    parser.add_option("-p", "--profile", dest="profile", help="profile to use with nightlies", metavar="PATH")
    parser.add_option("-a", "--args", dest="cmdargs", help="command-line arguments to pass to the application", metavar="ARG1,ARG2", default="")
    (options, args) = parser.parse_args()

    addons = strsplit(options.addons, ",")
    cmdargs = strsplit(options.cmdargs, ",")
    
    if not options.good_date:
        options.good_date = "2008-01-01"
        print "No 'good' date specified, using " + options.good_date

    runner = NightlyRunner(addons=addons, profile=options.profile, cmdargs=cmdargs)
    bisector = Bisector(runner)
    bisector.bisect(get_date(options.good_date), get_date(options.bad_date))


if __name__ == "__main__":
    cli()



import re
import httplib2
import datetime
import platform

def current_platform():
    (bits, linkage) = platform.architecture()

    os = platform.system()
    if os == 'Microsoft' or os == 'Windows' or re.match(".*cygwin.*", os):
        return "Windows " + bits # 'Windows 32bit'
    elif os == 'Linux':
        return "Linux " + bits
    elif os == 'Darwin' or os == 'Mac':
        return "Mac " + bits

def strsplit(string, sep):
    strlist = string.split(sep)
    if len(strlist) == 1 and strlist[0] == '': # python's split function is ridiculous
      return []
    return strlist
    
def download_url(url, dest=None):
    h = httplib2.Http()
    resp, content = h.request(url, "GET")
    if dest == None:
        dest = os.path.basename(url)

    local = open(dest, 'wb')
    local.write(content)
    local.close()
    return dest

def get_date(dateString):
    p = re.compile('(\d{4})\-(\d{1,2})\-(\d{1,2})')
    m = p.match(dateString)
    if not m:
        print "Incorrect date format"
        return
    return datetime.date(int(m.group(1)), int(m.group(2)), int(m.group(3)))


#!/usr/bin/env python
import os
import datetime
import httplib2
import re
import sys
import platform
import subprocess
from optparse import OptionParser
from BeautifulSoup import BeautifulSoup

try:
  from mozprofile import FirefoxProfile
except:
  from mozrunner import FirefoxProfile

from mozrunner import Runner
from mozInstall import MozInstaller
from mozInstall import rmdirRecursive
from utils import strsplit, download_url, get_date, current_platform 

class Nightly(object):
    def download(self, date=datetime.date.today(), dest=None):
        url = self.getBuildUrl(date)
        if url:
            if not dest:
                dest = os.path.basename(url)
            print "\nDownloading nightly...\n"  #TODO: doesn't belong here
            download_url(url, dest)
            self.dest = dest
            return True
        else:
            return False

    def install(self):
        rmdirRecursive("moznightlyapp")
        subprocess._cleanup = lambda : None # mikeal's fix for subprocess threading bug
        MozInstaller(src=self.dest, dest="moznightlyapp")
    

class FirefoxNightly(Nightly):
    def __init__(self, platform=current_platform()):
        self.profileClass = FirefoxProfile

        if platform == "Windows 64bit":
            print "No nightly builds available for 64 bit Windows"
            sys.exit()
        if platform == "Windows" or platform == "Windows 32bit":
            self.buildRegex = ".*win32.zip"
            self.processName = "firefox.exe"
            self.executablePath = "moznightlyapp/firefox/firefox.exe"
        elif platform == "Linux" or platform == "Linux 32bit":
            self.buildRegex = ".*linux-i686.tar.bz2"
            self.processName = "firefox-bin"
            self.executablePath = "moznightlyapp/firefox/firefox"
        elif platform == "Linux 64bit":
            self.buildRegex = ".*linux-x86_64.tar.bz2"
            self.processName = "firefox-bin"
            self.executablePath = "moznightlyapp/firefox/firefox"
        elif platform == "Mac" or platform=="Mac 32bit" or platform == "Mac 64bit":
            self.buildRegex = ".*mac.dmg"
            self.processName = "firefox-bin"
            self.executablePath = "moznightlyapp/Minefield.app/Contents/MacOS/firefox-bin"

    def getBuildUrl(self, date):
        # we don't know which hour the build was made, so look through all of them
        for i in [3, 2, 4, 5, 6, 1, 0] + range(7, 23):
            url = self.getUrl(date, i)
            if url:
                return url
                
    def formatDatePart(self, part):
        if part < 10:
            part = "0" + str(part)
        return str(part)

    def getUrl(self, date, hour):
        url = "http://ftp.mozilla.org/pub/mozilla.org/firefox/nightly/"
        year = str(date.year)
        month = self.formatDatePart(date.month)
        day = self.formatDatePart(date.day)
        url += year + "/" + month + "/" + year + "-" + month + "-" + day + "-"
        url += self.formatDatePart(hour) + "-" + self.getFirefoxTrunk(date) + "/"

        # now parse the page for the correct build url
        h = httplib2.Http();
        resp, content = h.request(url, "GET")
        if resp.status != 200:
            return False
        soup = BeautifulSoup(content)
        for link in soup.findAll('a'):
            href = link.get("href")
            if re.match(self.buildRegex, href):     
                return url + href

    def getFirefoxTrunk(self, date):
        if date < datetime.date(2008, 06, 17):
            return "trunk"
        else:
            return "mozilla-central"

    def getAppInfo(self):
        appIni = open(os.path.join(os.path.dirname(self.executablePath), "application.ini"), 'r')
        appInfo = appIni.read()
        appIni.close()
        repo = re.search('^SourceRepository\=(.+)', appInfo, re.M).group(1)
        changeset = re.search('^SourceStamp\=(.+)', appInfo, re.M).group(1)
        return (repo, changeset)


class NightlyRunner(object):
    def __init__(self, addons=None, application=FirefoxNightly(current_platform()), profile=None, cmdargs=[]):
        self.addons = addons
        self.profile = profile
        self.application = application
        self.cmdargs = cmdargs

    def start(self, date=datetime.date.today()):
        if not self.application.download(date=date):
            print "could not find nightly from " + str(date)
            return False # download failed
        self.application.install()

        if self.profile:
            profile = self.application.profileClass(profile=self.profile, create_new=False, addons=self.addons)
        elif len(self.addons):
            profile = self.application.profileClass(addons=self.addons)
        else:
            profile = self.application.profileClass()

        print "running Firefox nightly from " + str(date) + "\n"
        self.runner = Runner(binary=self.application.executablePath, cmdargs=self.cmdargs, profile=profile)
        self.runner.names = [self.application.processName]
        self.runner.start()
        return True

    def stop(self):
        self.runner.stop()

    def getAppInfo(self):
        return self.application.getAppInfo()
        
def cli():
    parser = OptionParser()
    parser.add_option("-d", "--date", dest="date", help="date of the nightly", metavar="YYYY-MM-DD", default=str(datetime.date.today()))
    parser.add_option("-a", "--addons", dest="addons", help="list of addons to install", metavar="PATH1,PATH2", default="")
    parser.add_option("-p", "--profile", dest="profile", help="path to profile to user", metavar="PATH")
    (options, args) = parser.parse_args()

    runner = NightlyRunner(addons=strsplit(options.addons, ","), profile=options.profile)
    runner.start(get_date(options.date))


if __name__ == "__main__":
    cli()
